import Dispatcher from "events-dispatch";

class KeyboardListener {
  constructor(period) {
    new Dispatcher(this);
    this.actions = [];
    this.setListeners();
    this.timer = null;
    this.timerInterval = period || 100; // ms
  }

  setListeners() {
    document.addEventListener('keydown', (e) => {
      let code = e.keyCode;
      let action = KeyboardListener.codes[code];
      if (action) {
        
        let index = this.actions.indexOf(action);
        if (index === -1) {
          this.actions.push(action);
          this.trigger(action);
        }
        if (!this.started) this.start();
      }
    });
    document.addEventListener('keyup', e => {
      let code = e.keyCode;
      let action = KeyboardListener.codes[code];
      if (action) {
        let index = this.actions.indexOf(action);
        if (index !== -1) this.actions.splice(index, 1);
        if (this.actions.length == 0) this.stop();
      }
    })
  }

  start() {
    this.started = true;
    let launch = () => {
      this.actions.forEach(action => {
        this.trigger(action);
      });
    }
    this.timer = setInterval(launch, this.timerInterval);
    launch();
  }

  stop() {
    this.started = false;
    clearInterval(this.timer);
  }
}

KeyboardListener.codes = {
  39: "RIGHT",
  68: "RIGHT",
  102: "RIGHT",

  37: "LEFT",
  65: "LEFT",
  100: "LEFT",

  40: "DOWN",
  83: "DOWN",
  98: "DOWN",

  38: "UP",
  87: "UP",
  104: "UP",

  13: "ENTER",
  32: "SPACE",
  8: "BACKSPACE",
  46: "DELETE"
};

export default KeyboardListener;